# pylint: disable=missing-class-docstring

"""
Decorate string types with some classes.
"""


class VideoId(str):
    pass


class ChannelId(str):
    pass


class ChannelName(str):
    pass


class ChannelUrl(str):
    pass
